package ru.tularegion.corp

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationRail
import androidx.compose.material3.NavigationRailItem
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.navigation.NavDestination.Companion.hierarchy
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.currentBackStackEntryAsState
import androidx.navigation.compose.rememberNavController
import corp_tularegion_extension.composeapp.generated.resources.Res
import corp_tularegion_extension.composeapp.generated.resources.round_favorite_24
import corp_tularegion_extension.composeapp.generated.resources.tularegion_48
import corp_tularegion_extension.composeapp.generated.resources.round_info_24
import corp_tularegion_extension.composeapp.generated.resources.round_open_in_new_24
import corp_tularegion_extension.composeapp.generated.resources.round_settings_24
import org.jetbrains.compose.resources.DrawableResource
import org.jetbrains.compose.resources.painterResource
import ru.tularegion.corp.backend.openCorpPortal
import ru.tularegion.corp.ui.screens.AboutScreen
import ru.tularegion.corp.ui.screens.LinksScreen
import ru.tularegion.corp.ui.screens.SettingsScreen
import ru.tularegion.corp.ui.theme.darkScheme

enum class Screen(
    val route: String,
    val title: String,
    val icon: DrawableResource,
) {
    Links(
        route = "links",
        title = "Быстрые ссылки",
        icon = Res.drawable.round_favorite_24,
    ),
    Settings(
        route = "settings",
        title = "Настройки",
        icon = Res.drawable.round_settings_24,
    ),
    About(
        route = "about",
        title = "О приложении",
        icon = Res.drawable.round_info_24,
    ),
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun App() {
    val navController = rememberNavController()
    val navBackStackEntry = navController.currentBackStackEntryAsState()
    val floatingActionButtonState = remember {
        mutableStateOf<(@Composable () -> Unit)?>(null)
    }

    MaterialTheme(
        colorScheme = darkScheme,
    ) {
        Surface {
            Row {
                NavigationRail(
                    containerColor = MaterialTheme.colorScheme.surfaceContainerLow,
                ) {
                    Column(
                        modifier = Modifier
                            .padding(
                                vertical = 8.dp,
                            )
                            .fillMaxHeight(),
                        verticalArrangement = Arrangement.SpaceBetween,
                        horizontalAlignment = Alignment.CenterHorizontally,
                    ) {
                        Column(
                            horizontalAlignment = Alignment.CenterHorizontally,
                        ) {
                            Spacer(
                                modifier = Modifier
                                    .height(8.dp),
                            )
                            Image(
                                painter = painterResource(Res.drawable.tularegion_48),
                                contentDescription = "Герб Тульской области",
                            )
                            Spacer(
                                modifier = Modifier
                                    .height(24.dp),
                            )
                            Screen.entries.forEach { screen ->
                                NavigationRailItem(
                                    selected = navBackStackEntry.value?.destination?.hierarchy?.any {
                                        it.route == screen.route
                                    } == true,
                                    onClick = {
                                        navController.navigate(screen.route)
                                    },
                                    icon = {
                                        Icon(
                                            painter = painterResource(screen.icon),
                                            contentDescription = "Открыть вкладку «${screen.title}»",
                                        )
                                    },
                                )
                            }
                        }
                        NavigationRailItem(
                            selected = false,
                            onClick = {
                                openCorpPortal()
                            },
                            icon = {
                                Icon(
                                    painter = painterResource(Res.drawable.round_open_in_new_24),
                                    contentDescription = "Открыть Корпоративный портал",
                                )
                            },
                        )
                    }
                }

                Scaffold(
                    topBar = {
                        TopAppBar(
                            title = {
                                Text(
                                    text = Screen.entries.find {
                                        it.route == navBackStackEntry.value?.destination?.route
                                    }?.title ?: "",
                                )
                            },
                        )
                    },
                    floatingActionButton = {
                        floatingActionButtonState.value?.let {
                            it()
                        }
                    },
                ) { paddingValues ->
                    NavHost(
                        navController = navController,
                        startDestination = Screen.Links.route,
                        modifier = Modifier
                            .verticalScroll(
                                state = rememberScrollState(),
                            )
                            .padding(paddingValues)
                            .padding(
                                bottom = 16.dp,
                            )
                            .fillMaxSize(),
                    ) {
                        composable(Screen.Links.route) {
                            LinksScreen(
                                floatingActionButtonState = floatingActionButtonState,
                            )
                        }
                        composable(Screen.Settings.route) {
                            SettingsScreen()
                        }
                        composable(Screen.About.route) {
                            AboutScreen()
                        }
                    }
                }
            }
        }
    }
}